@@ -2,18 +2,19 @@ use serde::{Deserialize, Serialize};
 use tui_prompts::State as _;
 
 use crate::{
-    items::TargetData, menu::Menu, prompt::PromptData, state::State, term::Term, CmdLogEntry, Res,
+    cmd_log::CmdLogEntry, items::TargetData, menu::Menu, prompt::PromptData, state::State,
+    term::Term, Res,
 };
 use std::{
     ffi::{OsStr, OsString},
     fmt::Display,
     process::Command,
     rc::Rc,
-    sync::{Arc, RwLock},
 };
 
 pub(crate) mod checkout;
 pub(crate) mod commit;
+pub(crate) mod copy_hash;
 pub(crate) mod discard;
 pub(crate) mod editor;
 pub(crate) mod fetch;
@@ -76,6 +77,7 @@ pub(crate) enum Op {
     Unstage,
     Show,
     Discard,
+    CopyHash,
 
     ToggleSection,
     MoveUp,
@@ -140,6 +142,7 @@ impl Op {
             Op::Show => Box::new(show::Show),
             Op::Stage => Box::new(stage::Stage),
             Op::Unstage => Box::new(unstage::Unstage),
+            Op::CopyHash => Box::new(copy_hash::CopyHash),
         }
     }
 }
@@ -171,25 +174,24 @@ pub(crate) fn cmd_arg(command: fn(&OsStr) -> Command, arg: OsString) -> Action {
 }
 
 pub(crate) fn create_y_n_prompt(mut action: Action, prompt: &'static str) -> Action {
-    let update_fn =
-        Rc::new(move |state: &mut State, term: &mut Term| {
-            if state.prompt.state.status().is_pending() {
-                match state.prompt.state.value() {
-                    "y" => {
-                        Rc::get_mut(&mut action).unwrap()(state, term)?;
-                        state.prompt.reset(term)?;
-                    }
-                    "" => (),
-                    _ => {
-                        state.current_cmd_log_entries.push(Arc::new(RwLock::new(
-                            CmdLogEntry::Error("Aborted".to_string()),
-                        )));
-                        state.prompt.reset(term)?;
-                    }
+    let update_fn = Rc::new(move |state: &mut State, term: &mut Term| {
+        if state.prompt.state.status().is_pending() {
+            match state.prompt.state.value() {
+                "y" => {
+                    Rc::get_mut(&mut action).unwrap()(state, term)?;
+                    state.prompt.reset(term)?;
+                }
+                "" => (),
+                _ => {
+                    state
+                        .current_cmd_log
+                        .push(CmdLogEntry::Error("Aborted".to_string()));
+                    state.prompt.reset(term)?;
                 }
             }
-            Ok(())
-        });
+        }
+        Ok(())
+    });
 
     Rc::new(move |state: &mut State, _term: &mut Term| {
         state.prompt.set(PromptData {
@@ -201,12 +203,20 @@ pub(crate) fn create_y_n_prompt(mut action: Action, prompt: &'static str) -> Act
     })
 }
 
-pub(crate) fn create_rev_prompt(
+pub(crate) fn create_prompt(
     prompt: &'static str,
     callback: fn(&mut State, &mut Term, &[OsString], &str) -> Res<()>,
+) -> Action {
+    create_prompt_with_default(prompt, callback, |_| None)
+}
+
+pub(crate) fn create_prompt_with_default(
+    prompt: &'static str,
+    callback: fn(&mut State, &mut Term, &[OsString], &str) -> Res<()>,
+    default_fn: fn(&State) -> Option<String>,
 ) -> Action {
     Rc::new(move |state: &mut State, _term: &mut Term| {
-        let prompt_text = if let Some(default) = default_rev(state) {
+        let prompt_text = if let Some(default) = default_fn(state) {
             format!("{} (default {}):", prompt, default).into()
         } else {
             format!("{}:", prompt).into()
@@ -221,14 +231,14 @@ pub(crate) fn create_rev_prompt(
                     let input = state.prompt.state.value().to_string();
                     state.prompt.reset(term)?;
 
-                    let default_rev = default_rev(state);
-                    let rev = match (input.as_str(), &default_rev) {
+                    let default_value = default_fn(state);
+                    let value = match (input.as_str(), &default_value) {
                         ("", None) => "",
-                        ("", Some(default)) => default,
+                        ("", Some(selected)) => selected,
                         (value, _) => value,
                     };
 
-                    callback(state, term, &args, rev)?;
+                    callback(state, term, &args, value)?;
                 }
                 Ok(())
             }),
@@ -237,7 +247,7 @@ pub(crate) fn create_rev_prompt(
     })
 }
 
-fn default_rev(state: &State) -> Option<String> {
+pub(crate) fn selected_rev(state: &State) -> Option<String> {
     match &state.screen().get_selected_item().target_data {
         Some(TargetData::Branch(branch)) => Some(branch.to_owned()),
         Some(TargetData::Commit(commit)) => Some(commit.to_owned()),
